// Communications avec AUTOMGEN

#include "windows.h"
#include "a7com.h"


// Le numro de client (par dfaut 2 : 0 pour AUTOMGEN, 1 pour IRIS)
unsigned short CLIENT=2;

// Le numro d'instance
unsigned short INSTANCE=0;

// Le point d'entre des fonctions de ASYS
void (*AsysEntryPoint)(struct _asys_packet *asys_packet);

HINSTANCE hAsys;

// Se connecter  ASYS
BOOL ConnectToAsys(LPTSTR asysname)
{
#ifdef AUTOMGEN8
hAsys=LoadLibraryA(asysname?asysname:"ASYS8.DLL");
#else
hAsys=LoadLibrary(asysname?asysname:L"ASYS7.DLL");
#endif
if(hAsys==(HINSTANCE)NULL) 
	{ // Impossible de trouver ASYS.DLL (probablement pas accessible dans un des rpertoires de recherche)
	return FALSE;
	}
(FARPROC)AsysEntryPoint=GetProcAddress(hAsys,"AsysEntryPoint");
if((void *)AsysEntryPoint==NULL)
	{ // Ne devrait jamais arriver : impossible de trouver le point d'entre dans ASYS.DLL
	FreeLibrary(hAsys);
	(void *)hAsys=NULL;
	return FALSE;
	}
return TRUE;
}

// Se dconnecter de ASYS
void DeconnectAsys(void)
{
if(hAsys!=(HINSTANCE)NULL) FreeLibrary(hAsys);
hAsys=NULL;
}


BOOL bReadAsysProcess;

// Retourne le numro et le type d'une variable  partir d'une chane
// contenant un nom de variable AUTOMGEN sans espace et en majuscule.
// En retour TRUE si nom de variable ok, FALSE sinon
BOOL ParseVarName(LPSTR var,PBYTE pbType,PWORD pwNumber)
{
switch(*var)
	{
	case 'i':
	case 'I' : *pbType=VAR_I; break; 
	case 'o':
	case 'O' : *pbType=VAR_O; break; 
	case 'u' :
	case 'U' :
	case 'b' :
	case 'B' : *pbType=VAR_B; break; 
	case 't' :
	case 'T' : *pbType=VAR_T; break; 
	case 'c' :
	case 'C' : *pbType=VAR_C; break; 
	case 'm' :
	case 'M' : *pbType=VAR_M; break; 
	case 'f' :
	case 'F' : *pbType=VAR_F; break; 
	case 'l' :
	case 'L' : *pbType=VAR_L; break; 
	default : return FALSE;
	}
var++;
if(!isdigit(*var)) return FALSE;
*pwNumber=(unsigned short)atoi(var);
if(*pwNumber>9999) return FALSE;
var++;
while(isdigit(*var)) var++;
if(*var) return FALSE;
return TRUE;
}

WORD wNumRead;
BYTE cTypeRead;
WORD wNumWrite;
BYTE cTypeWrite;

#pragma pack(1)
struct _asys_packet asys_packet;
#pragma pack()

// Extraire la rponse aprs une lecture de variable
BOOL ReadAsysGetAnswer(LPDWORD pdwResult,PWORD pwStatus)
{
// Dpiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=1|(CLIENTWRITEONEMESS<<8);
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd==(WORD)-1) 
	{
	*pwStatus=(WORD)-1;
	return FALSE; // Rponse pas disponible
	}
*pwStatus=(unsigned short)(asys_packet.wAx>>8);
// Suivant le type de variable le rsultat est prsent dans diffrents menbres de asys_packet
switch(cTypeRead)
	{
	case VAR_I:
	case VAR_O:
	case VAR_B:
		{
		*pdwResult=((asys_packet.wAx&0xff)==0)?1:0;
		} break;
	case VAR_C:
	case VAR_M:
		{
		*pdwResult=(DWORD)asys_packet.wCx;
		} break;
	case VAR_T:
	case VAR_F:
	case VAR_L:
		{
		*pdwResult=MAKELONG(asys_packet.wCx,asys_packet.wDx);
		} break;
	}
return TRUE;
}

// Lire une variable : le nom de la variable au format AUTOMGEN est une chane de caractres dans 'var'
BOOL ReadAsysEx(LPSTR var,BOOL bWaitAnswer,LPDWORD pdwResult,PWORD pwStatus)
{
BYTE cType;
WORD wNum;
if(!ParseVarName(var,&cType,&wNum)) 
	{
	*pwStatus=(WORD)-1;
	return FALSE; // Erreur sur le nom de la variable
	}
return ReadAsys(cType,wNum,bWaitAnswer,pdwResult,pwStatus);
}

// Lire une variable en fournissant le type et le numro
BOOL ReadAsys(BYTE cType,WORD wNum,BOOL bWaitAnswer,LPDWORD pdwResult,PWORD pwStatus)
{
cTypeRead=cType;
wNumRead=wNum;
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTWRITEONEMESS<<8;
asys_packet.wAx=(WORD)cTypeRead;
asys_packet.wBx=wNumRead;
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
if(bWaitAnswer)
	{
	DWORD dwTime;
	dwTime=GetTickCount();
	do
		{
		if(ReadAsysGetAnswer(pdwResult,pwStatus)) return TRUE;
		Sleep(100);
		}
	while(GetTickCount()-dwTime<TIMOUT);
	*pwStatus=(WORD)-2;
	return FALSE; // Timout
	}
return TRUE;
}



// Extraire la rponse aprs une criture de variable
BOOL WriteAsysGetAnswer(PWORD pwStatus)
{
// Dpiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=1|(CLIENTWRITEONEMESS<<8);
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd==(WORD)-1) 
	{
	*pwStatus=(WORD)-1;
	return FALSE; // Rponse pas disponible
	}
*pwStatus=(unsigned short)(asys_packet.wAx>>8);
return TRUE;
}

// Ecrire une variable : le nom de la variable au format AUTOMGEN est une chane de caractres dans 'var'
BOOL WriteAsysEx(LPSTR var,BOOL bWaitAnswer,DWORD dwVal,PWORD pwStatus)
{
BYTE cType;
WORD wNum;
if(!ParseVarName(var,&cType,&wNum)) 
	{
	*pwStatus=(WORD)-1;
	return FALSE; // Erreur sur le nom de la variable
	}
return WriteAsys(cType,wNum,bWaitAnswer,dwVal,pwStatus);
}

BOOL WriteAsys(BYTE cType,WORD wNum,BOOL bWaitAnswer,DWORD dwVal,PWORD pwStatus)
{
cTypeWrite=cType;
wNumWrite=wNum;
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTWRITEONEMESS<<8;
asys_packet.wAx=0x100;
asys_packet.wBx=wNumWrite;
asys_packet.wCx=(WORD)cTypeWrite;
// Suivant le type de variable la valeur est  placer dans diffrents menbres de asys_packet
switch(cTypeWrite)
	{
	case VAR_I:
	case VAR_O:
	case VAR_B:
		{
		// Grer l'inversion des variables boolennes
		if(dwVal==0xffffffff) asys_packet.wCx|=0x500;
		else
			{
			asys_packet.wCx|=((unsigned char)dwVal)<<8;
			}
		} break;
	case VAR_C:
	case VAR_M:
	case VAR_T:
		{
		asys_packet.wDx=(WORD)dwVal;
		} break;
	case VAR_L:
	case VAR_F:
		{
		WORD w;
		asys_packet.wDx=LOWORD(dwVal);
		w=HIWORD(dwVal);
		asys_packet.wAx|=w&0xff;
		asys_packet.wCx|=w&0xff00;
		} break;
	}

(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
if(bWaitAnswer)
	{
	DWORD dwTime;
	dwTime=GetTickCount();
	do
		{
		if(WriteAsysGetAnswer(pwStatus)) return TRUE;
		Sleep(100);
		}
	while(GetTickCount()-dwTime<TIMOUT);
	*pwStatus=(WORD)-2;
	return FALSE; // Timout
	}
return TRUE;
}

// Test si un excuteur est install
BOOL IsExecutorAsys(void)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTWRITEONEMESS<<8;
asys_packet.wAx=0;
asys_packet.wBx=(WORD)-1;
asys_packet.wCx=(WORD)-1;
asys_packet.wDx=(WORD)-1;
(*AsysEntryPoint)(&asys_packet);
return asys_packet.wCmd==0?TRUE:FALSE;
}

// Extraire la rponse aprs un changement de mode
BOOL SetModeAsysGetAnswer(PWORD pwStatus)
{
// Dpiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=1|(CLIENTWRITEONEMESS<<8);
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd==(WORD)-1) 
	{
	*pwStatus=(WORD)-1;
	return FALSE; // Rponse pas disponible
	}
*pwStatus=(unsigned short)(asys_packet.wAx>>8);
return TRUE;
}

// Change le mode de marche
BOOL SetModeAsys(BYTE cMode,BOOL bWaitAnswer,PWORD pwStatus)
{
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTWRITEONEMESS<<8;
switch(cMode)
	{
	case RUN : 
		{
		asys_packet.wAx=0x300;
		} break;
	case STOP : 
		{
		asys_packet.wAx=0x200;
		} break;
	case STEP : 
		{
		asys_packet.wAx=0x400;
		} break;
	case INIT :
		{
		asys_packet.wAx=0x500;
		} break;
	default : 
		{
		*pwStatus=(WORD)-3; // Mode invalide
		return FALSE; // Timout
		}
	}
asys_packet.wBx=0;
asys_packet.wCx=0;
asys_packet.wDx=0;
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
if(bWaitAnswer)
	{
	DWORD dwTime;
	dwTime=GetTickCount();
	do
		{
		if(SetModeAsysGetAnswer(pwStatus)) return TRUE;
		Sleep(100);
		}
	while(GetTickCount()-dwTime<TIMOUT);
	*pwStatus=(WORD)-2;
	return FALSE; // Timout
	}
return TRUE;
}

// Change le numro de client
void SetClient(WORD wClient)
{
CLIENT=wClient;
}

// Change le numro d'instance
void SetInstance(WORD wInstance)
{
INSTANCE=wInstance;
}

// Initialise le systme de requtes multiples
void MultipleReadAsysInit(void)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTINITSTACK<<8;
(*AsysEntryPoint)(&asys_packet);
}

void MultipleWriteAsysCheck(PWORD pwStatus)
{
MultipleReadAsysCheck(pwStatus);
}

// Initialise le systme de requtes multiples
void MultipleWriteAsysInit(void)
{
MultipleReadAsysInit();
}
// Empile une requte
BOOL MultipleWriteAsysPush(BYTE cType,WORD wNum,DWORD dwVal,PWORD pwStatus)
{
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTPUSH<<8;
asys_packet.wAx=0x100;
asys_packet.wBx=wNum;
asys_packet.wCx=(WORD)cType;
// Suivant le type de variable la valeur est  placer dans diffrents menbres de asys_packet
switch(cType)
	{
	case VAR_I:
	case VAR_O:
	case VAR_B:
		{
		// Grer l'inversion des variables boolennes
		if(dwVal==0xffffffff) asys_packet.wCx|=0x500;
		else
			{
			if((dwVal&0xffff0000)==0xffff0000)
				{
				asys_packet.wCx=(WORD)(dwVal<<8);
				}
			else
				{
				if(dwVal!=0) asys_packet.wCx|=0x100;
				}
			}
		} break;
	case VAR_C:
	case VAR_M:
		{
		asys_packet.wDx=(WORD)dwVal;
		} break;
	case VAR_L:
	case VAR_F:
	case VAR_T:
		{
		WORD w;
		asys_packet.wDx=LOWORD(dwVal);
		w=HIWORD(dwVal);
		asys_packet.wAx|=w&0xff;
		asys_packet.wCx|=w&0xff00;
		} break;
	}
AsysEntryPoint(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
return TRUE;
}

// Dpile une requte
BOOL MultipleWriteAsysPop(BYTE cType,WORD wNum,PWORD pwStatus)
{
// Dpiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTPOP<<8;
AsysEntryPoint(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
*pwStatus=(unsigned short)asys_packet.wAx;
return TRUE;
}

// Empile une requte
BOOL MultipleReadAsysPush(BYTE cType,WORD wNum,PWORD pwStatus)
{
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTPUSH<<8;
asys_packet.wAx=(WORD)cType;
asys_packet.wBx=wNum;
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
return TRUE;
}

// Dpile une requte
BOOL MultipleReadAsysPop(BYTE cType,PWORD pwStatus,PDWORD pdwResult)
{
// Empiler la demande
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTPOP<<8;
(*AsysEntryPoint)(&asys_packet);
if(asys_packet.wCmd!=0)
	{
	*pwStatus=(WORD)asys_packet.wCmd;
	return FALSE;
	}
*pwStatus=(unsigned short)(asys_packet.wAx>>8);
// Suivant le type de variable le rsultat est prsent dans diffrents menbres de asys_packet
switch(cType)
	{
	case VAR_I:
	case VAR_O:
	case VAR_B:
		{
		*pdwResult=((asys_packet.wAx&0xff)==0)?1:0;
		} break;
	case VAR_C:
	case VAR_M:
		{
		*pdwResult=(DWORD)asys_packet.wCx;
		} break;
	case VAR_T:
	case VAR_F:
	case VAR_L:
		{
		*pdwResult=MAKELONG(asys_packet.wCx,asys_packet.wDx);
		} break;
	}
return TRUE;
}

// Test la pile des requtes
void MultipleReadAsysCheck(PWORD pwStatus)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=CLIENTCHECKSTACK<<8;
(*AsysEntryPoint)(&asys_packet);
*pwStatus=(WORD)asys_packet.wCmd;
}

#pragma pack(1)
// La structure passe avec le message EXECUTORISREADY
struct _asys_executorisready
	{
	LPVOID lpEntryPoint;	// Point d'entre de l'excuteur
	BOOL bUseSharedDatas;	// TRUE si les variables sont loges dans la mmoire partage de ASYS.DLL
	WORD wNinputs;			// Nombre d'entres
	WORD wNoutputs;			// Nombre de sorties
	WORD wNbits;			// Nombre de bits
	WORD wNtempos;			// Nombre de temporisations
	WORD wNcounts;			// Nombre de compteurs
	WORD wNwords;			// Nombre de mots
	WORD wNlongs;			// Nombre de longs
	WORD wNfloats;			// Nombre de flottants
	PBYTE pInputs;			// Pointeur sur les entres
	PBYTE pOutputs;			// Pointeur sur les sorties
	PBYTE pBits;			// Pointeur sur les bits
	PBYTE pTempos;			// Pointeur sur les temporisations
	PWORD pCounts;			// Pointeur sur les compteurs
	PWORD pWords;			// Pointeur sur les mot
	PDWORD pLongs;			// Pointeur sur les longs
	float *pFloats;			// Pointeur sur les flottants
	DWORD ProcessId;		// Identificateur du process de l'excuteur
	BOOL bOldExec;			// TRUE pour un excuteur 16 bits
	} asys_executorisready;	 

#pragma pack()

// Retourne un pointeur vers les variables partages ou NULL si impossible
LPVOID GetSharedDataPointerAsys(BYTE cType,PWORD pwLen)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=GETEXECUTOROPTIONS<<8;
asys_packet.lpExtra=(LPVOID)&asys_executorisready;
(*AsysEntryPoint)(&asys_packet);
if(!asys_executorisready.bUseSharedDatas) return NULL;
switch(cType)
	{
	case VAR_I :
		{
		*pwLen=(unsigned short)asys_executorisready.wNinputs;
		return asys_executorisready.pInputs;
		} 
	case VAR_O :
		{
		*pwLen=(unsigned short)asys_executorisready.wNoutputs;
		return asys_executorisready.pOutputs;
		} 
	case VAR_B :
		{
		*pwLen=(unsigned short)asys_executorisready.wNbits;
		return asys_executorisready.pBits;
		} 
	case VAR_T :
		{
		*pwLen=(unsigned short)asys_executorisready.wNtempos;
		return asys_executorisready.pTempos;
		} 
	case VAR_C :
		{
		*pwLen=(unsigned short)asys_executorisready.wNcounts;
		return asys_executorisready.pCounts;
		} 
	case VAR_M :
		{
		*pwLen=(unsigned short)asys_executorisready.wNwords;
		return asys_executorisready.pWords;
		} 
	case VAR_L :
		{
		*pwLen=(unsigned short)asys_executorisready.wNlongs;
		return asys_executorisready.pLongs;
		} 
	case VAR_F :
		{
		*pwLen=(unsigned short)asys_executorisready.wNfloats;
		return asys_executorisready.pFloats;
		} 
	}
return NULL;
}


WORD ReadSharedData(BYTE cType,WORD wNumber,DWORD *dwResult)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=READSHAREDDATA<<8;
asys_packet.wAx=cType;
asys_packet.wBx=wNumber;
(*AsysEntryPoint)(&asys_packet);
*dwResult=MAKELONG(asys_packet.wCx,asys_packet.wDx);
return asys_packet.wAx;
}

WORD WriteSharedData(BYTE cType,WORD wNumber,DWORD dwVal)
{
memset(&asys_packet,0,sizeof(asys_packet));
asys_packet.wInst=INSTANCE;
asys_packet.wClient=CLIENT;
asys_packet.wCmd=WRITESHAREDDATA<<8;
asys_packet.wAx=cType;
asys_packet.wBx=wNumber;
asys_packet.wCx=LOWORD(dwVal);
asys_packet.wDx=HIWORD(dwVal);
(*AsysEntryPoint)(&asys_packet);
return asys_packet.wAx;
}